#!/bin/bash
#Joel Bruner (aka brunerd)

#for debugging output: touch /tmp/debug
#to turn off: rm /tmp/debug
[ -f /tmp/debug ] && set -x

#############
# VARIABLES #
#############
#can specify space or comma separated locales
templateLocaleList="$4"

JSSDockRemediationFlag=1

#############
# FUNCTIONS #
#############

function getTemplateDockList
{
local thisLocale;
local dockList_CSV;
local thisDockPath;
local templateLocaleList="$1"

#if nothing is specified do ALL template folders
if [ -z "$templateLocaleList" ]; then
	#a way to get all the lproj locales in User Template
	local templateLocaleList=$(ls -1d /System/Library/User\ Template/*lproj | tr \/ $'\n' | grep lproj)
fi

#can be just space seperated but just in case
IFS=$' ,\n\t'
#loop through one or more locales
for thisLocale in $templateLocaleList; do
	if [ -d "/System/Library/User Template/${thisLocale}" ] ;then
		#make a path to where the pref would live in the User Template
		thisDockPath="/System/Library/User Template/${thisLocale}/Library/Preferences/com.apple.dock.plist"
		#add this Dock to List
		dockList_CSV+="$thisDockPath,"
	else
		echo "[NO_LOCALE] ${thisLocale}" > /dev/stderr
	fi
	
done
#return list of User Template Docks
echo "$dockList_CSV"
}

function ensureDockUserTemplateExists
{
local dockList_CSV="$1"
local thisDockPath;

#set IFS to recognize commas and omit spaces
IFS=$',\n\t'
for thisDockPath in $dockList_CSV; do
	#if localized user template does not exist for some reason, let's copy it in
	if [ ! -f "$thisDockPath" ]; then				
		#get the local from the path
		thisLocale=$(tr \/ $'\n' <<< "$thisDockPath" | grep lproj)
	
		#translate some ye olde User Template locales to the modern two letter xx.lproj format
		case $thisLocale in
		Dutch.lproj)
		thisLocale=nl.lproj
		;;
		English.lproj)
		thisLocale=en.lproj
		;;
		French.lproj)
		thisLocale=fr.lproj
		;;
		German.lproj)
		thisLocale=de.lproj
		;;
		Italian.lproj)
		thisLocale=it.lproj
		;;
		Japanese.lproj)
		thisLocale=ja.lproj
		;;
		Spanish.lproj)
		thisLocale=es.lproj
		;;
		esac

		#copy in default dock from Dock.app
		dockAppDefaultPath=/System/Library/CoreServices/Dock.app/Contents/Resources/"$thisLocale"/default.plist
		if [ -f "$dockAppDefaultPath" ] ;then
			echo "[COPYING] $dockAppDefaultPath to $thisDockPath"
			cp "$dockAppDefaultPath" "$thisDockPath"
		else
			echo "[NOT FOUND] $dockAppDefaultPath"
		fi
	fi
done
}


function jssDockRemediator 
{
#the dock file
local dockFilePath="$1"

#when the item is found set this
local itemPosition;
#tally up the total number of item
local itemCount;

echo "[EXAMINING] $dockFilePath"

#look through both dock categories (right and left side)
for itemCategory in persistent-apps persistent-others; do
	#just in case a loop occurs let's limit it to 100 items
	for (( dockItemID=0; dockItemID < 100; dockItemID++ )); do 

		#get dockItem name from _CFURLString
		local dockItem=$(/usr/libexec/PlistBuddy "$dockFilePath" -c "print $itemCategory:$dockItemID:tile-data:file-data:_CFURLString" 2> /dev/null);
		exitCode=$?

		#if empty try reading 'home directory relative' (used by template default.plist)
		if [ -z "$dockItem" ]; then
			#test and pipe stderr to stdout for tesing
			dockItem=$(/usr/libexec/PlistBuddy "$dockFilePath" -c "print $itemCategory:$dockItemID:tile-data:'home directory relative'" 2>&1);
			exitCode=$?
		fi

		#might be at the end so return now if that's the case
		#PlistBuddy used to not return error codes so the double brackets let's us test for wildcard strings
		if [ -z "$dockItem" -o "$exitCode" -ne 0 ] || [[ "$dockItem" == *Does\ Not\ Exist ]]; then 
			break;
		fi

		#JSS Dock Item flaw, we must ensure a GUID exists for first element other Dock is obliterated do all the be sure
		#value can be 0, Dock.app will be correct and make unique
		local GUID=$(/usr/libexec/PlistBuddy "$dockFilePath" -c "print $itemCategory:$dockItemID:GUID" 2>/dev/null)
		if [ -z "$GUID" ]; then
			echo "[JSSFIX] GUID 0 added to item $dockItem ($itemCategory:$dockItemID)"
			/usr/libexec/PlistBuddy "$dockFilePath" -c "add $itemCategory:$dockItemID:GUID integer 0"
		fi
	
		local fileLabel=$(/usr/libexec/PlistBuddy "$dockFilePath" -c "print $itemCategory:$dockItemID:tile-data:file-label" 2>/dev/null)
		#write label if the label is empty and not just ~
		if [ -z "$fileLabel" -a "$dockItem" != '~' ]; then
		local fileBaseName="$(basename "$dockItem")"
		fileLabel="${fileBaseName%.*}"
			echo "[JSSFIX] Missing File Label \"$fileLabel\" added to item $dockItem ($itemCategory:$dockItemID)"
			/usr/libexec/PlistBuddy "$dockFilePath" -c "add $itemCategory:$dockItemID:tile-data:file-label string $fileLabel"
		fi		
	done
done
}

##########
## MAIN ##
##########

#get list of all paths to where all User Template com.apple.dock.plist would live
dockList_CSV="$(getTemplateDockList "$templateLocaleList")"
#ensure they all exist, copying in the default.plist from Dock.app if necessary
ensureDockUserTemplateExists "$dockList_CSV"

#hopefully we can remove this one day
#currently the Dock Item code on the JSS makes assumptions on the com.apple.dock.plist that aren't true if it is the default.plist from Dock.app
#If a GUID is not present on the 1st elements of persistent-others and persistent-apps it will be overwritten completely
#file-label is required to avoid duplication, even though _CFURLString would be a truly unique value
#since template plists have never been touch by Dock.app they lack both these things - we can "fix" that
if [ "$JSSDockRemediationFlag" -eq 1 ]; then
	#set IFS to recognize commas and omit spaces
	IFS=$',\n\t'

	#go through each dock plist
	for dockFilePath in $dockList_CSV; do
		jssDockRemediator "$dockFilePath"
	done
fi
